"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Query = void 0;
const helper_1 = require("./helper");
const lodash_1 = __importDefault(require("lodash"));
class Query {
    constructor(prisma) {
        this.prisma = prisma;
        this.sheets = [];
        this.keys = {};
        this.result = {
            body: []
        };
        this.points = [];
        this.grid = {};
        this.allSheets = [];
        this.tags = [];
        return this;
    }
    setGrid(grid) {
        this.grid = grid;
        return this;
    }
    get(tags) {
        return __awaiter(this, void 0, void 0, function* () {
            const allTagIds = (tags[0].hasOwnProperty('category'))
                ? [tags] : tags;
            for (const level in allTagIds) {
                const tagIds = allTagIds[level];
                const sheets = yield this.getSheets(tagIds);
                this.processSheets(sheets, Number(level));
            }
            this.setDataPointKeys();
            return this;
        });
    }
    getByIds(allTagIds) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const level in allTagIds) {
                const tagIds = allTagIds[level];
                const selectionTags = yield this.getTagInfo(tagIds);
                this.tags.push(selectionTags);
                const sheets = yield this.getSheetsByTags(selectionTags);
                this.processSheets(sheets, Number(level));
            }
            this.setDataPointKeys();
            return this;
        });
    }
    processSheets(sheets, level) {
        const dataPoints = [];
        if (sheets.length > 0) {
            this.parseSheets(sheets);
            this.setDataPoints(dataPoints);
            this.modifyDataPoints(dataPoints, level);
        }
        this.pushDataPoints(dataPoints);
    }
    getTagInfo(tagIds) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.prisma.tag.findMany({
                where: {
                    id: {
                        in: tagIds
                    }
                }
            });
        });
    }
    getSheets(tagStrings) {
        return __awaiter(this, void 0, void 0, function* () {
            const dataTags = yield this.getTags(tagStrings);
            const tags = [];
            dataTags.forEach(dataTag => {
                const tag = {
                    id: dataTag.id,
                    name: dataTag.value,
                    categoryId: dataTag.categoryId
                };
                tags.push(tag);
            });
            return this.findSheets(tags);
        });
    }
    getSheetsByTags(tags) {
        return __awaiter(this, void 0, void 0, function* () {
            const sheets = yield this.findSheets(tags);
            return sheets;
        });
    }
    getSheetsById(ids) {
        return __awaiter(this, void 0, void 0, function* () {
            const selectionTags = yield this.prisma.tag.findMany({
                where: {
                    id: {
                        in: ids
                    }
                }
            });
            const sheets = yield this.findSheets(selectionTags);
            return sheets;
        });
    }
    findSheets(tags) {
        return __awaiter(this, void 0, void 0, function* () {
            let clause = helper_1.getNestedClause(tags);
            if (!clause)
                return [];
            let sheets = yield this.prisma.sheet.findMany({
                where: clause,
                include: {
                    tags: true
                }
            });
            return sheets;
        });
    }
    parseSheets(sheets) {
        this.sheets = sheets.map(sheet => {
            return {
                id: sheet.id,
                tags: sheet.tags.map(tag => {
                    return {
                        id: tag.id,
                        value: tag.name,
                        categoryId: tag.categoryId
                    };
                }),
                rows: JSON.parse(sheet.rows),
                columns: JSON.parse(sheet.columns),
                data: JSON.parse(sheet.data),
                meta: JSON.parse(sheet.meta),
            };
        });
        this.allSheets.push(...this.sheets);
    }
    setDataPoints(dataPoints) {
        this.sheets.forEach(sheet => {
            sheet.data.forEach((points, r) => {
                points.forEach((point, c) => {
                    dataPoints.push({
                        tags: sheet.tags,
                        row: sheet.rows[r],
                        column: sheet.columns[c],
                        value: point,
                        meta: this.getDataPointMeta(sheet.meta, r, c)
                    });
                });
            });
        });
        return dataPoints;
    }
    getDataPointMeta(meta, r, c) {
        helper_1.vd(meta);
        return null;
    }
    modifyDataPoints(dataPoints, level) {
        const modifiers = this.getDatapointModifiersByLevel(level);
        modifiers.forEach(modifier => {
            modifier.cb(dataPoints);
        });
    }
    getDatapointModifiersByLevel(level) {
        var _a, _b;
        const modifiers = [];
        (_b = (_a = this.grid) === null || _a === void 0 ? void 0 : _a.modify) === null || _b === void 0 ? void 0 : _b.forEach(modifier => {
            if (modifier.applyToLevel && modifier.applyToLevel.indexOf(level) > -1) {
                modifiers.push(modifier);
            }
        });
        return modifiers;
    }
    pushDataPoints(dataPoints) {
        this.points.push(...dataPoints);
    }
    setDataPointKeys() {
        this.sheets.forEach(sheet => {
            sheet.tags.forEach(tag => {
                if (tag.categoryId) {
                    this.addKey(String(tag.categoryId), tag.value);
                }
            });
            sheet.data.forEach((points, r) => {
                points.forEach((point, c) => {
                    this.addKey('row', sheet.rows[r]);
                    this.addKey('column', sheet.columns[c]);
                });
            });
        });
    }
    addKey(category, value) {
        if (!this.keys[category]) {
            this.keys[category] = {};
        }
        this.keys[category][value] = true;
    }
    merge() {
        let rows = this.checkForCallback(this.grid.rows, this.keys);
        let columns = this.checkForCallback(this.grid.columns, this.keys);
        let points = [];
        let result = {};
        rows.forEach((rowCb, r) => {
            let rowCbResult = rowCb(this.points);
            points[r] = rowCbResult.points;
            let rowKey = rowCbResult.label;
            result[rowKey] = {};
            columns.forEach((columnCb, c) => {
                let cellPoints = columnCb(points[r]);
                let colKey = cellPoints.label;
                result[rowKey][colKey] = this.grid.cell(cellPoints.points);
            });
        });
        this.setResult(result);
        return this;
    }
    clone() {
        return lodash_1.default.cloneDeep(this);
    }
    checkForCallback(cb, keys) {
        return (typeof cb === 'function')
            ? cb(keys)
            : cb;
    }
    getTagIds(tags) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let i in tags) {
                let tag = tags[i];
                yield this.setCategoryId(tag);
                yield this.setTagId(tag);
            }
            return tags.map(tag => tag.id);
        });
    }
    getTags(tags) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let i in tags) {
                let tag = tags[i];
                yield this.setCategoryId(tag);
                yield this.setTagId(tag);
            }
            return tags;
        });
    }
    setCategoryId(tag) {
        return __awaiter(this, void 0, void 0, function* () {
            if (tag.categoryId)
                return;
            const categoryItem = yield this.prisma.category.findFirst({
                where: {
                    name: tag.category
                }
            });
            if (!categoryItem) {
                throw new Error(`Category not found: ${tag.category}`);
            }
            tag.categoryId = categoryItem.id;
        });
    }
    setTagId(tag) {
        return __awaiter(this, void 0, void 0, function* () {
            if (tag.id)
                return;
            const tagItem = yield this.prisma.tag.findFirst({
                where: {
                    categoryId: tag.categoryId,
                    name: tag.value
                }
            });
            if (!tagItem) {
                throw new Error(`Tag not found: ${tag.value} @ category: ${tag.category}`);
            }
            tag.id = tagItem.id;
        });
    }
    setResult(result) {
        for (const r in result) {
            const cols = [];
            for (const c in result[r]) {
                cols.push({
                    key: c,
                    value: result[r][c]
                });
            }
            this.result.body.push({
                key: r,
                cols: cols
            });
        }
    }
    filterColumns(columns) {
        let targetColumns = [];
        targetColumns = (typeof (columns) !== 'object') ? [columns] : columns;
        this.result.body.forEach((row, r) => {
            if (row.cols) {
                this.result.body[r].cols = row.cols.filter((col, c) => {
                    return targetColumns.indexOf(c) >= 0;
                });
            }
        });
        return this;
    }
    sort(cb) {
        this.result.body.sort((a, b) => cb(a, b));
        return this;
    }
    toSeriesCategories() {
        if (this.result.body[0]) {
            const series = this.result.body[0].cols.map(col => { return { label: col.key }; });
            const categories = [];
            this.result.body.forEach(row => {
                categories.push({
                    label: row.key,
                    values: row.cols.map(cell => { return cell.value; })
                });
            });
            return {
                series: series,
                categories: categories
            };
        }
        return {
            series: [],
            categories: []
        };
    }
    toVerticalLines() {
        const series = this.result.body[0].cols.map(col => { return { label: col.key }; });
        const categories = [];
        this.result.body.forEach((row, r) => {
            categories.push({
                label: row.key,
                y: r,
                values: row.cols.map(cell => { return cell.value; })
            });
        });
        return {
            series: series,
            categories: categories
        };
    }
    toLabels() {
        return {
            body: this.result.body.map(row => {
                return {
                    values: [row.key]
                };
            })
        };
    }
    toRowLabels() {
        return this.toLabels();
    }
    toColumnLabels() {
        const series = this.result.body[0].cols.map(col => col.key);
        return {
            body: [
                {
                    values: series
                }
            ]
        };
    }
    toTable() {
        return {
            body: this.result.body.map(row => {
                return {
                    values: row.cols.map(cell => { return cell.value; })
                };
            }),
        };
    }
}
exports.Query = Query;
//# sourceMappingURL=query.js.map