"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Store = void 0;
const query_1 = require("./query");
class Store {
    constructor(prisma, options) {
        this.prisma = prisma;
        this.summary = {
            ids: [],
            deleted: []
        };
        this.query = new query_1.Query(this.prisma);
        this.options = {
            replaceExisting: true
        };
        if (options) {
            this.options = Object.assign(this.options, options);
        }
        this.importId = 0;
        this.categories = [];
        this.tags = [];
    }
    run(datasheets) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if ((_a = this.options) === null || _a === void 0 ? void 0 : _a.runBefore) {
                yield ((_b = this.options) === null || _b === void 0 ? void 0 : _b.runBefore(this.prisma));
            }
            yield this.getImport();
            yield this.getCategories();
            yield this.getTags();
            for (let i in datasheets) {
                yield this.storeData(datasheets[i]);
            }
            yield this.prisma.$disconnect();
            return this.summary;
        });
    }
    getImport() {
        return __awaiter(this, void 0, void 0, function* () {
            const newImport = yield this.prisma.import.create({
                data: {
                    file: 'Test',
                    user: {
                        connect: { id: 1 }
                    }
                }
            });
            this.importId = newImport.id;
        });
    }
    getCategories() {
        return __awaiter(this, void 0, void 0, function* () {
            this.categories = yield this.prisma.category.findMany();
        });
    }
    createCategory(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const newCat = yield this.prisma.category.create({
                data: {
                    name: name,
                    color: 'grey',
                    icon: 'settings'
                }
            });
            this.categories.push(newCat);
            return newCat;
        });
    }
    getTags() {
        return __awaiter(this, void 0, void 0, function* () {
            this.tags = yield this.prisma.tag.findMany();
        });
    }
    createTag(name, catId) {
        return __awaiter(this, void 0, void 0, function* () {
            const newTag = yield this.prisma.tag.create({
                data: {
                    name: name,
                    categoryId: catId,
                }
            });
            this.tags.push(newTag);
            return newTag;
        });
    }
    storeData(datasheet) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            yield this.addCategoryIdToTags(datasheet.tags);
            yield this.addTagIdsToTags(datasheet.tags);
            if ((_a = this.options) === null || _a === void 0 ? void 0 : _a.replaceExisting) {
                yield this.deleteExistingDatasheets(datasheet.tags);
            }
            yield this.createSheet(datasheet);
        });
    }
    addCategoryIdToTags(tags) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let i in tags) {
                const tag = tags[i];
                const cat = this.categories.find((cat) => cat.name === tag.category);
                if (cat && cat.id) {
                    tag.categoryId = cat.id;
                }
                else {
                    const newCat = yield this.createCategory(tag.category);
                    tag.categoryId = newCat.id;
                }
            }
        });
    }
    addTagIdsToTags(tags) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let i in tags) {
                const tag = tags[i];
                const existingTag = this.tags.find((existingTag) => existingTag.name === tag.value
                    && existingTag.categoryId === tag.categoryId);
                if (!existingTag) {
                    let newTag = yield this.createTag(tag.value, tag.categoryId);
                    tag.id = newTag.id;
                }
                else {
                    tag.id = existingTag.id;
                }
            }
        });
    }
    createSheet(datasheet) {
        return __awaiter(this, void 0, void 0, function* () {
            const newSheet = yield this.prisma.sheet.create({
                data: {
                    columns: JSON.stringify(datasheet.columns),
                    rows: JSON.stringify(datasheet.rows),
                    data: JSON.stringify(datasheet.data),
                    meta: JSON.stringify(datasheet.meta),
                    tags: {
                        connect: datasheet.tags.map(tag => {
                            return { id: tag.id };
                        })
                    },
                    import: {
                        connect: { id: this.importId }
                    }
                }
            });
            if (newSheet.id) {
                datasheet.id = newSheet.id;
                this.summary.ids.push(datasheet.id);
            }
            else {
                console.error(datasheet);
                throw new Error('Could not store datasheet.');
            }
            return newSheet;
        });
    }
    deleteExistingDatasheets(tags) {
        return __awaiter(this, void 0, void 0, function* () {
            const existingSheets = yield this.query.getSheets(tags);
            if (existingSheets.length) {
                const ids = existingSheets.map(sheet => sheet.id);
                yield this.prisma.sheet.deleteMany({
                    where: {
                        id: {
                            in: ids
                        }
                    }
                });
                this.summary.deleted.push(...ids);
            }
        });
    }
}
exports.Store = Store;
//# sourceMappingURL=store.js.map