"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const helper_1 = require("./helper");
const points_1 = __importDefault(require("./points"));
const lodash_1 = __importDefault(require("lodash"));
const resultInfo_1 = __importDefault(require("./helper/resultInfo"));
const transformResult_1 = __importDefault(require("./helper/transformResult"));
class Query {
    constructor(prisma) {
        this.prisma = prisma;
        this.sheets = [];
        this.inputKeys = {};
        this.keys = {};
        this.visibleKeys = {
            row: [],
            column: []
        };
        this.result = {
            body: []
        };
        this.points = [];
        this.grid = {};
        this.selectionValidator = () => true;
        this.allSheets = [];
        this.tags = [];
        this.nonGreedySelector = true;
        return this;
    }
    setGrid(grid) {
        this.grid = grid;
        return this;
    }
    setOptions(options) {
        if (options === null || options === void 0 ? void 0 : options.selectionValidator) {
            this.setSelectionValidator(options.selectionValidator);
        }
        if (options === null || options === void 0 ? void 0 : options.nonGreedySelector) {
            this.nonGreedySelector = options === null || options === void 0 ? void 0 : options.nonGreedySelector;
        }
        return this;
    }
    setSelectionValidator(validator) {
        this.selectionValidator = validator;
    }
    get(tags) {
        return __awaiter(this, void 0, void 0, function* () {
            const allTagIds = (tags[0].hasOwnProperty('category'))
                ? [tags] : tags;
            for (const level in allTagIds) {
                const tagIds = allTagIds[level];
                const sheets = yield this.getSheets(tagIds);
                this.processSheets(sheets, Number(level));
            }
            this.setDataPointKeys(this.points, 'keys');
            return this;
        });
    }
    getByIds(allTagIds) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const level in allTagIds) {
                const tagIds = allTagIds[level];
                const selectionTags = yield this.getTagInfo(tagIds);
                const isValid = this.selectionValidator(selectionTags);
                if (isValid) {
                    this.tags.push(selectionTags);
                    const sheets = yield this.getSheetsByTags(selectionTags);
                    this.processSheets(sheets, Number(level));
                }
            }
            this.setDataPointKeys(this.points, 'keys');
            return this;
        });
    }
    processSheets(sheets, level) {
        const dataPoints = [];
        if (sheets.length > 0) {
            this.parseSheets(sheets);
            this.setDataPoints(dataPoints);
            this.setDataPointKeys(dataPoints, 'inputKeys');
            this.modifyDataPoints(dataPoints, level);
        }
        this.pushDataPoints(dataPoints);
    }
    getTagInfo(tagIds) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.prisma.tag.findMany({
                where: {
                    id: {
                        in: tagIds
                    }
                }
            });
        });
    }
    getSheets(tagStrings) {
        return __awaiter(this, void 0, void 0, function* () {
            const dataTags = yield this.getTags(tagStrings);
            const tags = [];
            dataTags.forEach(dataTag => {
                const tag = {
                    id: dataTag.id,
                    name: dataTag.value,
                    categoryId: dataTag.categoryId
                };
                tags.push(tag);
            });
            return this.findSheets(tags);
        });
    }
    getSheetsByTags(tags) {
        return __awaiter(this, void 0, void 0, function* () {
            const sheets = yield this.findSheets(tags);
            return sheets;
        });
    }
    getSheetsById(ids) {
        return __awaiter(this, void 0, void 0, function* () {
            const selectionTags = yield this.prisma.tag.findMany({
                where: {
                    id: {
                        in: ids
                    }
                }
            });
            const sheets = yield this.findSheets(selectionTags);
            return sheets;
        });
    }
    findSheets(tags) {
        return __awaiter(this, void 0, void 0, function* () {
            let clause = helper_1.getNestedClause(tags);
            if (!clause)
                return [];
            let sheets = yield this.prisma.sheet.findMany({
                where: clause,
                include: {
                    tags: true
                }
            });
            if (this.nonGreedySelector) {
                sheets = this.filterSheets(sheets);
            }
            return sheets;
        });
    }
    parseSheets(sheets) {
        this.sheets = sheets.map(sheet => {
            return {
                id: sheet.id,
                tags: sheet.tags.map(tag => {
                    return {
                        id: tag.id,
                        value: tag.name,
                        categoryId: tag.categoryId
                    };
                }),
                rows: JSON.parse(sheet.rows),
                columns: JSON.parse(sheet.columns),
                data: JSON.parse(sheet.data),
                meta: JSON.parse(sheet.meta),
            };
        });
        this.allSheets.push(...this.sheets);
    }
    /*
     * Filters sheets with least tag count by categoryId.
     * This will prevent mixing sheets with different set of tags.
     */
    filterSheets(sheets) {
        const nonGreedyIds = this.getSheetCategoryCount(sheets).map(count => count.sheetId);
        return sheets.filter(sheet => nonGreedyIds.includes((sheet.id)));
    }
    getSheetCategoryCount(sheets) {
        const categoryCount = [];
        sheets.forEach(sheet => {
            categoryCount.push({
                sheetId: sheet.id,
                categoryIds: sheet.tags.map(tag => tag.categoryId)
            });
        });
        categoryCount.sort((a, b) => a.categoryIds.length - b.categoryIds.length);
        const smallestCount = categoryCount[0].categoryIds.length;
        return categoryCount.filter(categoryCount => categoryCount.categoryIds.length <= smallestCount);
    }
    setDataPoints(dataPoints) {
        this.sheets.forEach(sheet => {
            sheet.data.forEach((points, r) => {
                points.forEach((value, c) => {
                    const dataPoint = {
                        tags: sheet.tags,
                        row: sheet.rows[r],
                        column: sheet.columns[c],
                        value: value,
                        meta: this.getDataPointMeta(sheet, r, c),
                        getMeta: () => undefined,
                        getTag: () => undefined,
                    };
                    dataPoint.getMeta = Query.getMetaCb(dataPoint);
                    dataPoint.getTag = Query.getTagCb(dataPoint);
                    dataPoints.push(dataPoint);
                });
            });
        });
        return dataPoints;
    }
    getDataPointMeta(sheet, r, c) {
        const pointMeta = [];
        sheet.meta.forEach((metaContent) => {
            if ((metaContent === null || metaContent === void 0 ? void 0 : metaContent.key) === 'nested') {
                this.pushPointNestedMeta(metaContent, pointMeta, sheet, r, c);
            }
            else if (metaContent === null || metaContent === void 0 ? void 0 : metaContent.data) {
                if (this.metaHasRows(metaContent === null || metaContent === void 0 ? void 0 : metaContent.data)) {
                    this.pushPointMeta(pointMeta, metaContent.key, metaContent.data[r][c]);
                }
                else {
                    this.pushPointMeta(pointMeta, metaContent.key, metaContent.data[c]);
                }
            }
            if ((metaContent === null || metaContent === void 0 ? void 0 : metaContent.info) && metaContent.label === sheet.rows[r]) {
                this.pushPointInfo(metaContent, pointMeta, sheet, r, c);
            }
        });
        return pointMeta;
    }
    pushPointInfo(metaContent, pointMeta, sheet, r, c) {
        var _a;
        const hasMetaContent = (_a = metaContent.info) === null || _a === void 0 ? void 0 : _a.find(meta => meta.value === sheet.columns[c]);
        if (hasMetaContent && hasMetaContent.info) {
            this.pushPointMeta(pointMeta, hasMetaContent.info, hasMetaContent.key);
        }
    }
    pushPointNestedMeta(metaContent, pointMeta, sheet, r, c) {
        if (metaContent.label === sheet.rows[r]) {
            const parentValues = [];
            metaContent.data.forEach((parentLabel) => {
                const parentRow = sheet.rows.indexOf(parentLabel);
                parentValues.push({
                    label: parentLabel,
                    value: sheet.data[parentRow][c]
                });
                if (parentLabel === metaContent.label) {
                    this.pushPointMeta(pointMeta, 'isParent', true);
                }
            });
            this.pushPointMeta(pointMeta, metaContent.key, parentValues);
        }
    }
    metaHasRows(metaData) {
        return (metaData[0] && Array.isArray(metaData[0]));
    }
    pushPointMeta(pointMeta, key, value) {
        pointMeta.push({
            key: key,
            value: value
        });
    }
    modifyDataPoints(dataPoints, level) {
        const modifiers = this.getDatapointModifiersByLevel(level);
        const points = new points_1.default(dataPoints);
        modifiers.forEach(modifier => {
            if (modifier.cb) {
                modifier.cb(points);
            }
            if (modifier.callbacks) {
                modifier.callbacks.forEach(callback => {
                    if (typeof callback === 'function') {
                        callback(points);
                    }
                });
            }
        });
    }
    getDatapointModifiersByLevel(level) {
        var _a, _b;
        const modifiers = [];
        (_b = (_a = this.grid) === null || _a === void 0 ? void 0 : _a.modify) === null || _b === void 0 ? void 0 : _b.forEach(modifier => {
            if (modifier.applyToLevel && modifier.applyToLevel.indexOf(level) > -1) {
                modifiers.push(modifier);
            }
            else if (!modifier.applyToLevel || modifier.applyToLevel.length === 0) {
                modifiers.push(modifier);
            }
        });
        return modifiers;
    }
    pushDataPoints(dataPoints) {
        this.points.push(...dataPoints);
    }
    setDataPointKeys(dataPoints, mode) {
        dataPoints.forEach((point, c) => {
            this.addKey('row', point.row, mode);
            this.addKey('column', point.column, mode);
            this.addNestedKeys(point, mode);
            point.tags.forEach((tag) => {
                if (tag.categoryId) {
                    this.addKey(String(tag.categoryId), tag.value, mode);
                }
            });
        });
    }
    addNestedKeys(point, mode) {
        var _a, _b;
        const hasNestedMeta = (_a = point === null || point === void 0 ? void 0 : point.meta) === null || _a === void 0 ? void 0 : _a.find((meta) => meta.key === 'nested');
        if (hasNestedMeta && Array.isArray(hasNestedMeta.value)) {
            (_b = hasNestedMeta === null || hasNestedMeta === void 0 ? void 0 : hasNestedMeta.value) === null || _b === void 0 ? void 0 : _b.forEach((nested) => {
                this.addKey('nested', nested.label, mode);
            });
        }
    }
    addKey(category, value, mode) {
        if (!this[mode][category]) {
            this[mode][category] = {};
        }
        this[mode][category][value] = true;
    }
    merge() {
        return __awaiter(this, void 0, void 0, function* () {
            let rows = this.checkForCallback(this.grid.row, this.keys, this.points);
            let columns = this.checkForCallback(this.grid.column, this.keys, this.points);
            let points = [];
            let result = {};
            rows.forEach((rowCb, r) => {
                let rowCbResult = rowCb(this.points);
                points[r] = rowCbResult.points;
                let rowKey = rowCbResult.label;
                result[rowKey] = {};
                columns.forEach((columnCb, c) => {
                    let cellPoints = columnCb(points[r]);
                    let colKey = cellPoints.label;
                    result[rowKey][colKey] = this.grid.cell(cellPoints.points);
                });
            });
            this.setResult(result);
            if (this.grid.sort) {
                this.sortResult(this.grid.sort);
            }
            if (this.grid.transform) {
                yield this.transformResult(this.grid.transform);
            }
            return this;
        });
    }
    clone() {
        return lodash_1.default.cloneDeep(this);
    }
    checkForCallback(cb, keys, points) {
        const cbResult = (typeof cb === 'function')
            ? cb(keys, points)
            : cb;
        return cbResult;
    }
    getTagIds(tags) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let i in tags) {
                let tag = tags[i];
                yield this.setCategoryId(tag);
                yield this.setTagId(tag);
            }
            return tags.map(tag => tag.id);
        });
    }
    getTags(tags) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let i in tags) {
                let tag = tags[i];
                yield this.setCategoryId(tag);
                yield this.setTagId(tag);
            }
            return tags;
        });
    }
    setCategoryId(tag) {
        return __awaiter(this, void 0, void 0, function* () {
            if (tag.categoryId)
                return;
            const categoryItem = yield this.prisma.category.findFirst({
                where: {
                    name: tag.category
                }
            });
            if (!categoryItem) {
                throw new Error(`Category not found: ${tag.category}`);
            }
            tag.categoryId = categoryItem.id;
        });
    }
    setTagId(tag) {
        return __awaiter(this, void 0, void 0, function* () {
            if (tag.id)
                return;
            const tagItem = yield this.prisma.tag.findFirst({
                where: {
                    categoryId: tag.categoryId,
                    name: tag.value
                }
            });
            if (!tagItem) {
                throw new Error(`Tag not found: ${tag.value} @ category: ${tag.category}`);
            }
            tag.id = tagItem.id;
        });
    }
    setResult(result) {
        for (const r in result) {
            const cols = [];
            this.visibleKeys.row.push(r);
            for (const c in result[r]) {
                this.visibleKeys.column.push(c);
                cols.push({
                    key: c,
                    value: result[r][c],
                    getPoint: Query.getPointCb(result[r][c])
                });
            }
            this.result.body.push({
                key: r,
                cols: cols,
                getColumn: Query.getColumnCb(cols)
            });
        }
        this.result.isValid = () => {
            if (this.result.body && this.result.body[0] && this.result.body[0].cols) {
                return true;
            }
            return false;
        };
        this.result.info = new resultInfo_1.default(this.result);
        this.result.transform = new transformResult_1.default(this.result);
        this.visibleKeys.column = [...new Set(this.visibleKeys.column)];
    }
    static getPointCb(points) {
        return (index) => {
            const point = points[index || 0];
            return point;
        };
    }
    static getColumnCb(cols) {
        return (colId) => {
            return cols[colId || 0];
        };
    }
    static getMetaCb(point) {
        return (key) => {
            if (point.meta) {
                return point.meta.find(meta => meta.key === key);
            }
        };
    }
    static getTagCb(point) {
        return (categoryId) => {
            if (point.tags) {
                return point.tags.find(meta => meta.categoryId === categoryId);
            }
        };
    }
    filterColumns(columns) {
        let targetColumns = [];
        targetColumns = (typeof (columns) !== 'object') ? [columns] : columns;
        this.result.body.forEach((row, r) => {
            if (row.cols) {
                this.result.body[r].cols = row.cols.filter((col, c) => {
                    return targetColumns.indexOf(c) >= 0;
                });
            }
        });
        return this;
    }
    sort(cb) {
        this.result.body.sort((a, b) => cb(a, b));
        return this;
    }
    sortResult(sortation) {
        try {
            sortation.forEach(sort => {
                if (sort.cb && typeof sort.cb === 'function') {
                    sort.cb(this.result, this.points);
                }
            });
        }
        catch (e) {
            throw e;
        }
    }
    transformResult(transformations) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.result.isValid()) {
                return;
            }
            try {
                for (const transform of transformations) {
                    if (transform.cb && typeof transform.cb === 'function') {
                        yield transform.cb(this.result, this.points);
                    }
                }
            }
            catch (e) {
                throw e;
            }
        });
    }
}
exports.default = Query;
//# sourceMappingURL=query.js.map